@@ -10,6 +10,7 @@ from __future__ import division
 import collections
 import contextlib
 import errno
+import glob
 import io
 import os
 import pprint
@@ -55,7 +56,7 @@ if LINUX:
 
 
 # =====================================================================
-# utils
+# --- utils
 # =====================================================================
 
 
@@ -141,7 +142,7 @@ def get_free_version_info():
 
 
 # =====================================================================
-# system virtual memory
+# --- system virtual memory
 # =====================================================================
 
 
@@ -375,7 +376,7 @@ class TestSystemVirtualMemory(unittest.TestCase):
 
 
 # =====================================================================
-# system swap memory
+# --- system swap memory
 # =====================================================================
 
 
@@ -437,7 +438,7 @@ class TestSystemSwapMemory(unittest.TestCase):
 
 
 # =====================================================================
-# system CPU
+# --- system CPU
 # =====================================================================
 
 
@@ -536,9 +537,31 @@ class TestSystemCPU(unittest.TestCase):
             self.assertIsNone(psutil._pslinux.cpu_count_physical())
             assert m.called
 
+    def test_cpu_freq_no_result(self):
+        with mock.patch("psutil._pslinux.glob.glob", return_value=[]):
+            self.assertIsNone(psutil.cpu_freq())
+
+    @unittest.skipIf(TRAVIS, "fails on Travis")
+    def test_cpu_freq_use_second_file(self):
+        # https://github.com/giampaolo/psutil/issues/981
+        def glob_mock(pattern):
+            if pattern.startswith("/sys/devices/system/cpu/cpufreq/policy"):
+                flags.append(None)
+                return []
+            else:
+                flags.append(None)
+                return orig_glob(pattern)
+
+        flags = []
+        orig_glob = glob.glob
+        with mock.patch("psutil._pslinux.glob.glob", side_effect=glob_mock,
+                        create=True):
+            assert psutil.cpu_freq()
+            self.assertEqual(len(flags), 2)
+
 
 # =====================================================================
-# system CPU stats
+# --- system CPU stats
 # =====================================================================
 
 
@@ -559,7 +582,7 @@ class TestSystemCPUStats(unittest.TestCase):
 
 
 # =====================================================================
-# system network
+# --- system network
 # =====================================================================
 
 
@@ -671,7 +694,7 @@ class TestSystemNetwork(unittest.TestCase):
 
 
 # =====================================================================
-# system disk
+# --- system disk
 # =====================================================================
 
 
@@ -829,7 +852,7 @@ class TestSystemDisks(unittest.TestCase):
 
 
 # =====================================================================
-# misc
+# --- misc
 # =====================================================================
 
 
@@ -1020,6 +1043,11 @@ class TestMisc(unittest.TestCase):
             t.stop()
 
 
+# =====================================================================
+# --- sensors
+# =====================================================================
+
+
 @unittest.skipUnless(LINUX, "LINUX only")
 @unittest.skipUnless(hasattr(psutil, "sensors_battery") and
                      psutil.sensors_battery() is not None,
@@ -1036,6 +1064,8 @@ class TestSensorsBattery(unittest.TestCase):
     @unittest.skipUnless(which("acpi"), "acpi utility not available")
     def test_power_plugged(self):
         out = sh("acpi -b")
+        if 'unknown' in out.lower():
+            return unittest.skip("acpi output not reliable")
         plugged = "Charging" in out.split('\n')[0]
         self.assertEqual(psutil.sensors_battery().power_plugged, plugged)
 
@@ -1166,8 +1196,9 @@ class TestSensorsBattery(unittest.TestCase):
             self.assertIsNone(psutil.sensors_battery().power_plugged)
             assert m.called
 
+
 # =====================================================================
-# test process
+# --- test process
 # =====================================================================
 
 
@@ -1287,13 +1318,6 @@ class TestProcess(unittest.TestCase):
             p.cmdline() == ['foo', 'bar', '']
             assert m.called
 
-    def test_io_counters_mocked(self):
-        with mock.patch('psutil._pslinux.open', create=True) as m:
-            self.assertRaises(
-                NotImplementedError,
-                psutil._pslinux.Process(os.getpid()).io_counters)
-            assert m.called
-
     def test_readlink_path_deleted_mocked(self):
         with mock.patch('psutil._pslinux.os.readlink',
                         return_value='/home/foo (deleted)'):
