@@ -92,9 +92,12 @@ class TestProcess(unittest.TestCase):
         reap_children()
 
     def test_pid(self):
-        self.assertEqual(psutil.Process().pid, os.getpid())
+        p = psutil.Process()
+        self.assertEqual(p.pid, os.getpid())
         sproc = get_test_subprocess()
         self.assertEqual(psutil.Process(sproc.pid).pid, sproc.pid)
+        with self.assertRaises(AttributeError):
+            p.pid = 33
 
     def test_kill(self):
         sproc = get_test_subprocess()
@@ -323,16 +326,22 @@ class TestProcess(unittest.TestCase):
     @skip_on_not_implemented(only_if=LINUX)
     def test_io_counters(self):
         p = psutil.Process()
+
         # test reads
         io1 = p.io_counters()
         with open(PYTHON, 'rb') as f:
             f.read()
         io2 = p.io_counters()
         if not BSD:
-            assert io2.read_count > io1.read_count, (io1, io2)
+            self.assertGreater(io2.read_count, io1.read_count)
             self.assertEqual(io2.write_count, io1.write_count)
-        assert io2.read_bytes >= io1.read_bytes, (io1, io2)
-        assert io2.write_bytes >= io1.write_bytes, (io1, io2)
+            if LINUX:
+                self.assertGreater(io2.read_chars, io1.read_chars)
+                self.assertEqual(io2.write_chars, io1.write_chars)
+        else:
+            self.assertGreaterEqual(io2.read_bytes, io1.read_bytes)
+            self.assertGreaterEqual(io2.write_bytes, io1.write_bytes)
+
         # test writes
         io1 = p.io_counters()
         with tempfile.TemporaryFile(prefix=TESTFILE_PREFIX) as f:
@@ -341,10 +350,21 @@ class TestProcess(unittest.TestCase):
             else:
                 f.write("x" * 1000000)
         io2 = p.io_counters()
-        assert io2.write_count >= io1.write_count, (io1, io2)
-        assert io2.write_bytes >= io1.write_bytes, (io1, io2)
-        assert io2.read_count >= io1.read_count, (io1, io2)
-        assert io2.read_bytes >= io1.read_bytes, (io1, io2)
+        self.assertGreaterEqual(io2.write_count, io1.write_count)
+        self.assertGreaterEqual(io2.write_bytes, io1.write_bytes)
+        self.assertGreaterEqual(io2.read_count, io1.read_count)
+        self.assertGreaterEqual(io2.read_bytes, io1.read_bytes)
+        if LINUX:
+            self.assertGreater(io2.write_chars, io1.write_chars)
+            self.assertGreaterEqual(io2.read_chars, io1.read_chars)
+
+        # sanity check
+        for i in range(len(io2)):
+            if BSD and i >= 2:
+                # On BSD read_bytes and write_bytes are always set to -1.
+                continue
+            self.assertGreaterEqual(io2[i], 0)
+            self.assertGreaterEqual(io2[i], 0)
 
     @unittest.skipUnless(LINUX or (WINDOWS and get_winver() >= WIN_VISTA),
                          'platform not supported')
@@ -756,10 +776,11 @@ class TestProcess(unittest.TestCase):
         self.assertEqual(real, os.getuid())
         # os.geteuid() refers to "effective" uid
         self.assertEqual(effective, os.geteuid())
-        # no such thing as os.getsuid() ("saved" uid), but starting
-        # from python 2.7 we have os.getresuid()[2]
+        # No such thing as os.getsuid() ("saved" uid), but starting
+        # from python 2.7 we have os.getresuid() which returns all
+        # of them.
         if hasattr(os, "getresuid"):
-            self.assertEqual(saved, os.getresuid()[2])
+            self.assertEqual(os.getresuid(), p.uids())
 
     @unittest.skipUnless(POSIX, 'POSIX only')
     def test_gids(self):
@@ -769,10 +790,11 @@ class TestProcess(unittest.TestCase):
         self.assertEqual(real, os.getgid())
         # os.geteuid() refers to "effective" uid
         self.assertEqual(effective, os.getegid())
-        # no such thing as os.getsgid() ("saved" gid), but starting
-        # from python 2.7 we have os.getresgid()[2]
+        # No such thing as os.getsgid() ("saved" gid), but starting
+        # from python 2.7 we have os.getresgid() which returns all
+        # of them.
         if hasattr(os, "getresuid"):
-            self.assertEqual(saved, os.getresgid()[2])
+            self.assertEqual(os.getresgid(), p.gids())
 
     def test_nice(self):
         p = psutil.Process()
@@ -792,11 +814,17 @@ class TestProcess(unittest.TestCase):
             finally:
                 p.nice(psutil.NORMAL_PRIORITY_CLASS)
         else:
+            first_nice = p.nice()
             try:
-                first_nice = p.nice()
+                if hasattr(os, "getpriority"):
+                    self.assertEqual(
+                        os.getpriority(os.PRIO_PROCESS, os.getpid()), p.nice())
                 p.nice(1)
                 self.assertEqual(p.nice(), 1)
-                # going back to previous nice value raises
+                if hasattr(os, "getpriority"):
+                    self.assertEqual(
+                        os.getpriority(os.PRIO_PROCESS, os.getpid()), p.nice())
+                # XXX - going back to previous nice value raises
                 # AccessDenied on OSX
                 if not OSX:
                     p.nice(0)
@@ -1148,7 +1176,9 @@ class TestProcess(unittest.TestCase):
                 return
         self.fail("num ctx switches still the same after 50.000 iterations")
 
-    def test_parent_ppid(self):
+    def test_ppid(self):
+        if hasattr(os, 'getppid'):
+            self.assertEqual(psutil.Process().ppid(), os.getppid())
         this_parent = os.getpid()
         sproc = get_test_subprocess()
         p = psutil.Process(sproc.pid)
